import React, { Component, MouseEvent } from 'react';
import { Auction } from './auction';


type ListProps = {
  auctions: ReadonlyArray<Auction>,
  onNewClick: () => void,
  onRefreshClick: () => void,
  onAuctionClick: (index: number) => void
};

type ListState = {
  now: number,  // current time when rendering
};


// Shows the list of all the auctions.
export class AuctionList extends Component<ListProps, ListState> {

  constructor(props: ListProps) {
    super(props);
    this.state = {now: Date.now()}
  }

  componentDidUpdate = (prevProps: ListProps): void => {
    if (prevProps !== this.props) {
      this.setState({now: Date.now()});  // Force a refresh
    }
  };

  render = (): JSX.Element => {
    return (
      <div>
        <h2>Current Auctions</h2>
        <ul>{this.renderAuctions()}</ul>
        <button type="button" onClick={this.doRefreshClick}>Refresh</button>
        <button type="button" onClick={this.doNewClick}>New Auction</button>
      </div>);
  };

  renderAuctions = (): JSX.Element[] => {
    const auctions: JSX.Element[] = [];
    // Inv: auctions = LI for each of auctions[0 .. i-1]
    for (let i = 0; i < this.props.auctions.length; i++) {
      const auction = this.props.auctions[i];
      const min = (auction.endTime - this.state.now) / 60 / 1000;
      const desc = (min < 0) ? "" :
          <span>&nbsp;&ndash; {Math.round(min)} minutes remaining</span>;
      auctions.push(
        <li key={auction.name}>
          <a href="#" onClick={(evt) => this.doAuctionClick(evt, i)}>{auction.name}</a>
          {desc}
        </li>);
    }
    return auctions;
  };

  doRefreshClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    this.props.onRefreshClick();
  };

  doNewClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    this.props.onNewClick();  // tell the parent to show the new auction page
  };

  doAuctionClick = (evt: MouseEvent<HTMLAnchorElement>, index: number): void => {
    evt.preventDefault();
    this.props.onAuctionClick(index);
  };
}
